-- v4 (2021-04-26)
-- added smoke spawning
--
-- v3:
-- - spawn neutral units
--
-- v2: ???
-- mehrere des selben types instanziieren
-- staffeln instanziieren
-- ground units SAM testen
-- enemy cap testen
-- korrektes loadout

local jgfZeus = {
    -- group name to group object
    database = {},
    id = 0
}

function jgfZeus:generateId()
    self.id = self.id + 1
    return self.id
end

function jgfZeus:generateSuffix()
    return "__jgfzeus_" .. tostring(self:generateId())
end


local initDatabase = function(mission) 
    
    local parseCoalition = function(coalition)
        for _, country in pairs(coalition) do
            --env.info("parsing country " .. to_string(country))
            local types = {
                "helicopter",
                "ship",
                "plane",
                "vehicle",
                "static"
            }
            
            for _, type in pairs(types) do
                local typedGroups = country[type]
                --env.info("parsing country ".. country.name .." typedGroups("  .. type .. ") " .. to_string(typedGroups))
                if typedGroups ~= nil and typedGroups.group ~= nil then
                    for _, group in pairs(typedGroups.group) do
                        --env.info("parsing group " .. to_string(group))
                        local groupCopy = deepcopy(group)
                        
                        -- save memory
                        if groupCopy.route and groupCopy.route.spans then 
                            groupCopy.route.spans = nil
                        end
                        
                        groupCopy.name = env.getValueDictByKey(groupCopy.name)
                        
                        for _, unit in pairs(groupCopy.units) do
                            unit.name = env.getValueDictByKey(unit.name)
                        end
                        
                        jgfZeus.database[groupCopy.name] = groupCopy
                    end
                end
            end
        end
    end

    parseCoalition(mission.coalition.blue.country)
    parseCoalition(mission.coalition.red.country)
    parseCoalition(mission.coalition.neutrals.country)

    --env.info("db " .. to_string(jgfZeus.database))
end

function table_print (tt, indent, done)
  done = done or {}
  indent = indent or 0
  if type(tt) == "table" then
    local sb = {}
    for key, value in pairs (tt) do
      table.insert(sb, string.rep (" ", indent)) -- indent it
      if type (value) == "table" and not done [value] then
        done [value] = true
        table.insert(sb, key .. " = {\n");
        table.insert(sb, table_print (value, indent + 2, done))
        table.insert(sb, string.rep (" ", indent)) -- indent it
        table.insert(sb, "}\n");
      elseif "number" == type(key) then
        table.insert(sb, string.format("\"%s\"\n", tostring(value)))
      else
        table.insert(sb, string.format(
            "%s = \"%s\"\n", tostring (key), tostring(value)))
       end
    end
    return table.concat(sb)
  else
    return tt .. "\n"
  end
end

function to_string( tbl )
    if  "nil"       == type( tbl ) then
        return tostring(nil)
    elseif  "table" == type( tbl ) then
        return table_print(tbl)
    elseif  "string" == type( tbl ) then
        return tbl
    else
        return tostring(tbl)
    end
end

function deepcopy(orig, copies)
    copies = copies or {}
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        if copies[orig] then
            copy = copies[orig]
        else
            copy = {}
            copies[orig] = copy
            for orig_key, orig_value in next, orig, nil do
                copy[deepcopy(orig_key, copies)] = deepcopy(orig_value, copies)
            end
            setmetatable(copy, deepcopy(getmetatable(orig), copies))
        end
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end

SupportHandler = {}

function handleSpawnRequest(text, coord)
    local group = jgfZeus.database[text]
    --env.info("spawning group")
    if group ~= nil then
        
        local category = Group.getByName(text):getCategory()
        local country = Group.getByName(text):getUnit(1):getCountry()
        local groupCopy = deepcopy(group)
        
        groupCopy.name = groupCopy.name .. jgfZeus:generateSuffix()
        groupCopy.lateActivation = false
		
		local firstPoint = groupCopy.route.points[1]
		firstPoint.x = coord.x
		firstPoint.y = coord.z
        
        local offsetX = 0
        for _,  unit in pairs(groupCopy.units) do
            unit.x = coord.x + offsetX
            unit.y = coord.z
            --env.info("unitX " .. tostring(unit.x) .. " unit.y" .. tostring(unit.y))
            unit.name = unit.name .. jgfZeus:generateSuffix()
            offsetX = offsetX + 25
        end

        --env.info("before add group " .. to_string(country) .. " " .. to_string(category) .. " " .. to_string(groupCopy))
        coalition.addGroup(country, category, groupCopy)
        --env.info("after spawn")
	elseif text == "greensmoke" then
	    trigger.action.smoke(coord, 0)
	elseif text == "redsmoke" then
	    trigger.action.smoke(coord, 1)
	elseif text == "whitesmoke" then
	    trigger.action.smoke(coord, 2)
	elseif text == "orangesmoke" then
	    trigger.action.smoke(coord, 3)
	elseif text == "bluesmoke" then
	    trigger.action.smoke(coord, 4)
    end
end

function markRemoved(event)
    if event.text~=nil and event.text:lower():find("-") then 
        local text = event.text:lower()
        
        local text = event.text
        local startIndex = text:find("-create ")
        --env.info("start index " .. tostring(startIndex))
        if startIndex == 1 then
            local unit = text:gsub("-create ", "")
            --env.info("try to spawn " .. tostring(unit))
            handleSpawnRequest(unit, event.pos)
        end
    end
end

function SupportHandler:onEvent(event)
    if event.id == world.event.S_EVENT_MARK_REMOVED then
        markRemoved(event)
    end
end


initDatabase(env.mission)

world.addEventHandler(SupportHandler)
